--[[

  HealBot Contined
	
]]

local _scale=0;
local CalcEquipBonus=false;
local FlagEquipUpdate1=1;
local FlagEquipUpdate2=1;
local NeedEquipUpdate=0;
local initSpells=true;
local HealValue=0;

function HealBot_AddChat(msg)
  local chanid=HealBot_Get_DebugChan();
  if chanid then
    local hour,minute = GetGameTime();
	if minute==0 then
      msg="["..hour..":00] "..msg;
    elseif minute<10 then
      msg="["..hour..":0"..minute.."] "..msg; 
    else
      msg="["..hour..":"..minute.."] "..msg; 
    end
    SendChatMessage(msg , "CHANNEL", nil, chanid); 
  end
end

HealBot_Debug = false;

function HealBot_AddDebug(msg)
  if HealBot_Debug then
   	HealBot_AddChat("HealBot_DEBUG: " .. msg);
  end
end

function HealBot_AddError(msg)
  UIErrorsFrame:AddMessage(msg, 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
  HealBot_AddChat(msg);
end

function HealBot_TogglePanel(panel)
  if (not panel) then return end
  if ( panel:IsVisible() ) then
    HideUIPanel(panel);
  else
    ShowUIPanel(panel);
  end
end

function HealBot_StartMoving(frame)
  if ( not frame.isMoving ) and ( frame.isLocked ~= 1 ) then
    frame:StartMoving();
    frame.isMoving = true;
  end
end

function HealBot_StopMoving(frame)
  if ( frame.isMoving ) then
    frame:StopMovingOrSizing();
    frame.isMoving = false;
  end
end

function HealBot_SlashCmd(cmd)
  if (cmd=="") then
    HealBot_TogglePanel(HealBot_Action);
    return
  end
  if (cmd=="options" or cmd=="opt" or cmd=="config" or cmd=="cfg") then
    HealBot_TogglePanel(HealBot_Options);
    return
  end
  if (cmd=="debug" or cmd=="dbg") then
    if HealBot_Debug then
      HealBot_AddDebug("Debug messages disabled");
      HealBot_Debug = false;
      HealBot_Clear_DebugChan();
    else
      HealBot_Debug = true;
      HealBot_AddDebug("Debug messages enabled");
      ShowUIPanel(HealBot_Action);
    end
    return;
  end
  if (cmd=="reset" or cmd=="recalc" or cmd=="defaults") then
--    HealBot_RecalcSpells();
    HealBot_Options_Defaults_OnClick(HealBot_Options_Defaults);
    return
  end
  if (cmd=="ui") then
    ReloadUI();
    return;
  end
  if (cmd=="init") then
  	HealBot_RegisterThis(this);
  end
  if (cmd=="x") then
    initSpells=true;
	NeedEquipUpdate=1
  	HealBot_RecalcSpells();
    return;
  end
  if (cmd=="ver") then
    SendAddonMessage( "HealBot", "RequestVersion", "RAID" );
    return;
  end
end

function HealBot_TargetName()
  if UnitIsEnemy("target","player") then return nil end
--  if not UnitPlayerControlled("target") then return nil end
  if (UnitIsPlayer("target")) then
    if UnitIsUnit("target","player") then return "player" end
    if (UnitInParty("target")) then 
      for i=1,4 do
        if UnitIsUnit("target","party"..i) then return "party"..i end
      end
    end
    if (UnitInRaid("target")) then 
      for i=1,40 do
        if UnitIsUnit("target","raid"..i) then return "raid"..i end
      end
    end
  else
    if UnitIsUnit("target","pet") then return "pet" end
    if (UnitInParty("player")) then 
      for i=1,4 do
        if UnitIsUnit("target","partypet"..i) then return "partypet"..i end
      end
    end
    if (UnitInRaid("player")) then 
      for i=1,40 do
        if UnitIsUnit("target","raidpet"..i) then return "raidpet"..i end
      end
    end
  end
  return nil
end

local HealBot_IsFighting = false;

function HealBot_PackBagSlot(bag,slot)
  return bag*100+slot;
end

function HealBot_UnpackBagSlot(bagslot)
  return math.floor(bagslot/100),math.mod(bagslot,100);
end

function HealBot_GetItemName(bag,slot)
  local link = GetContainerItemLink(bag,slot);
  if not link then return nil end;
  local _,_,item = string.find(link,"%[(.*)%]");
  local _,count = GetContainerItemInfo(bag,slot);
  return item,count;
end

function HealBot_GetBagSlot(item)
  local BagSlot,BestCount;
  for bag=0,NUM_BAG_FRAMES do
    for slot=1,GetContainerNumSlots(bag) do
      local bagitem,count = HealBot_GetItemName(bag,slot);
      if (item==bagitem) then
        if not BestCount or BestCount>count then
          BagSlot = HealBot_PackBagSlot(bag,slot);
          BestCount = count;
        end
      end
    end
  end
  return BagSlot;
end

function HealBot_UseItem(item)
  local bagslot = HealBot_GetBagSlot(item);
  if not bagslot then return end;
  local bag,slot = HealBot_UnpackBagSlot(bagslot);
  local Link = GetContainerItemLink(bag,slot);
  UseContainerItem(bag,slot);
end

function HealBot_GetSpellName(id)
  if (not id) then
    return nil;
  end
  local spellName, subSpellName = GetSpellName(id,BOOKTYPE_SPELL);
  if (not spellName) then
    return nil;
  end
  if (not subSpellName or subSpellName=="") then
    return spellName;
  end
  return spellName .. "(" .. subSpellName .. ")";
end

function HealBot_GetSpellId(spell)
  local id,idd = 1,0; 
  while true do 
    local spellName, subSpellName = GetSpellName(id,BOOKTYPE_SPELL);
    if (spellName) then
      if (spell == spellName .. "(" .. subSpellName .. ")") then
        return id;
      end
      if (spell == spellName) then
        idd=id;
      end   
    else
      do break end
    end
    id = id + 1;
  end
  if idd>0 then
    return idd
  else
    return nil;
  end
end

function HealBot_CastSpellByName(spell,target)
  if (HealBot_Spells[spell] and HealBot_Spells[spell].BagSlot) then
    HealBot_UseItem(spell);
    return;
  end
  local id;
  if HealBot_Spells[spell].id then
    id = HealBot_Spells[spell].id
  else
    id = HealBot_GetSpellId(spell);
    if (not id) then
      return;
    end
  end
  CastSpell(id,BOOKTYPE_SPELL);
end

HealBot_CastingSpell  = nil;
HealBot_CastingTarget = nil;

function HealBot_StartCasting(spell,target)
  HealBot_CastSpellByName(spell);
  HealBot_CastingSpell  = spell;
  HealBot_CastingTarget = target;
  
  if ( SpellIsTargeting() ) then 
    SpellTargetUnit(target);
  end
  if HealBot_Spells[spell].Cast > 1 then
    HealValue=HealBot_Spells[spell].HealsDur;
	SendAddonMessage( HEALBOT_ADDON_ID, ">> "..UnitName(target).." <<=>> "..HealValue.." << ", "RAID" );
  end
  if HealBot_CastNotify>1 then
    if target=="target" then target = HealBot_TargetName() or "target" end
    local Notify = HealBot_CastNotify;
    if Notify==5 and GetNumRaidMembers()==0 then Notify = 4 end
    if Notify==4 and GetNumPartyMembers()==0 then Notify = 3 end
    if Notify==3 and not (UnitPlayerControlled(target) and target~='player' and target~='pet') then Notify = 2 end
    if Notify==3 then
      SendChatMessage(string.format(HEALBOT_CASTINGSPELLONYOU,spell),"WHISPER",nil,UnitName(target));
    elseif Notify==4 then
      SendChatMessage(string.format(HEALBOT_CASTINGSPELLONUNIT,spell,UnitName(target)),"PARTY",nil,nil);
    elseif Notify==5 then
      SendChatMessage(string.format(HEALBOT_CASTINGSPELLONUNIT,spell,UnitName(target)),"RAID",nil,nil);
    else
      HealBot_AddChat(string.format(HEALBOT_CASTINGSPELLONUNIT,spell,UnitName(target)));
    end
  end
end

function HealBot_StopCasting()
  if HealBot_CastingTarget then
   if HealBot_HealsIn[UnitName(HealBot_CastingTarget)] then
    if HealValue > 0 then
	  SendAddonMessage( HEALBOT_ADDON_ID, ">> "..UnitName(HealBot_CastingTarget).." <<=>> "..0-HealValue.." << ", "RAID" );
      HealValue=0;
    end
   end
  end
  HealBot_CastingSpell  = nil;
  HealBot_CastingTarget = nil;
end

function HealBot_AbortCasting()
  HealBot_AbortButton=0;
  HealBot_Action_PartyChanged();
end

local HealBot_Health60 = {
  ["DRUID"]   = 3500,
  ["MAGE"]    = 2500,
  ["HUNTER"]  = 3500,
  ["PALADIN"] = 4000,
  ["PRIEST"]  = 2500,
  ["ROGUE"]   = 3500,
  ["SHAMAN"]  = 3800,
  ["WARLOCK"] = 3500,
  ["WARRIOR"] = 5000,
}
function HealBot_UnitHealth(unit)
  local Current,Desired = UnitHealth(unit),UnitHealthMax(unit);
  if unit=='target' and Desired==100 then
    local class,level = HealBot_UnitClass(unit),UnitLevel(unit);
    if HealBot_Health60[class] and level>0 then
      Desired = math.floor(HealBot_Health60[class]/60*level+0.5)
    else
      Desired = UnitHealthMax('player');
    end
    Current = Desired/100*Current;
  end
  return Current,Desired*HealBot_HealthPct;
end

function HealBot_CheckCasting(unit)
  if not HealBot_CastingSpell or HealBot_AlwaysHeal() then return nil end
  if not HealBot_Spells[HealBot_CastingSpell] then return nil end
  if not unit then unit = HealBot_CastingTarget end
  if unit~=HealBot_CastingTarget then return nil end

  if HealBot_IsCasting==false and HealBot_AbortButton==0 then
    HealBot_AbortButton=1;
    HealBot_Action_PartyChanged();
    return nil
  end

  local Current,Desired = HealBot_UnitHealth(unit)
  local Needed = Desired-Current;
  Needed = Needed * (1 + (HealBot_OverHeal*4));
  if Needed<0 then Needed = 0 end
  if (Needed>HealBot_Spells[HealBot_CastingSpell].HealsDur) then 
    HealBot_AbortButton=1;
    return nil 
  elseif HealBot_AbortButton==1 and HealBot_IsCasting==true then
    HealBot_AbortCasting();
  end
end

function HealBot_CastSpellOnTarget(spell,target)
  local old;
  if (not spell or not target or not UnitName(target)) then
    return;
  end
  if (UnitCanAttack("player","target")) then
    old = "enemy";
  else
    old = UnitName("target");
  end
  TargetUnit(target);
  HealBot_StartCasting(spell,target);
  if (old=="enemy") then
    TargetLastEnemy();
  elseif (old) then
    TargetByName(old);
  else
    ClearTarget();
  end
end

function HealBot_CastSpellOnFriend(spell,target)
  local old;
  if (not spell or not target or not UnitName(target)) then
    return;
  end
  if (UnitCanAttack("player","target")) then
    old = "enemy";
  else
    old = UnitName("target");
    TargetUnit(target);
  end
  HealBot_StartCasting(spell,target);
  if (old=="enemy") then
    TargetLastEnemy();
  elseif (old) then
    TargetByName(old);
  else
    ClearTarget();
  end
end

function HealBot_UnitClass(unit)
  local playerClass, englishClass = UnitClass(unit);
  return englishClass;
end

-- TBD: use the event UNIT_AURA to keep track instead of querying each time

function HealBot_UnitAffected(unit,effect)
  if not effect then return nil; end
  local i = 1
  while true do
    local buff = UnitBuff(unit,i)
    if not buff then
      do break end
    end
    if buff==effect then
      return buff
    end
    i = i + 1
  end
  i = 1
  while true do
    local debuff = UnitDebuff(unit,i)
    if not debuff then
      do break end
    end
    if debuff==effect then
      return debuff
    end
    i = i + 1
  end
  return nil;
end
-- safer to use GameTooltip:SetUnitBuff and read the lines in the tooltip ...
-- maybe make an additional GameTooltip frame if possible ?

function HealBot_SetItemDefaults(spell)
  if not HealBot_Spells[spell].Target then
    HealBot_Spells[spell].Target = {"player","party","pet"};
  end
  if not HealBot_Spells[spell].Price then
    HealBot_Spells[spell].Price = 0;
  end
  if not HealBot_Spells[spell].Cast then
    HealBot_Spells[spell].Cast = 0;
  end
  if not HealBot_Spells[spell].Mana then
    HealBot_Spells[spell].Mana = 0;
  end
  if not HealBot_Spells[spell].Channel then
    HealBot_Spells[spell].Channel = HealBot_Spells[spell].Cast;
  end
  if not HealBot_Spells[spell].Duration then
    HealBot_Spells[spell].Duration = HealBot_Spells[spell].Channel;
  end
  if not HealBot_Spells[spell].HealsMin then
    HealBot_Spells[spell].HealsMin = 0;
  end
  if not HealBot_Spells[spell].HealsMax then
    HealBot_Spells[spell].HealsMax = 0;
  end
  HealBot_Spells[spell].RealHealing=0;
  HealBot_Spells[spell].HealsCast = (HealBot_Spells[spell].HealsMin+HealBot_Spells[spell].HealsMax)/2;
  if not HealBot_Spells[spell].HealsExt then
    HealBot_Spells[spell].HealsExt = 0;
  end
end

function HealBot_SetSpellDefaults(spell)
  HealBot_Spells[spell].HealsDur = floor((HealBot_Spells[spell].HealsCast+HealBot_Spells[spell].HealsExt) + HealBot_Spells[spell].RealHealing);
end

function HealBot_AddHeal(spell)
  HealBot_SetSpellDefaults(spell);
  if HealBot_UseHealing[HealBot_Spells[spell].Group]==0 then return end
  table.foreachi(HealBot_Spells[spell].Target,function (i,val)
    table.insert(HealBot_Heals[val],spell);
  end);
  HealBot_Spells[spell].BagSlot = HealBot_GetBagSlot(spell);
end


function HealBot_FindHealSpells()
  local id = 1;
  local cnt = 0;
  HealBot_Heals = { player = {}, pet = {}, party = {} };
  if initSpells then
    cnt = HealBot_InitSpells();
    if cnt==0 then
	  return;
	else
      initSpells = false;
    end
  end
  table.foreach(HealBot_CurrentSpells, function (index,spell)
    if (HealBot_Spells[spell]) then
      if CalcEquipBonus then
        local healingbonus_penalty=1;
        if HealBot_Spells[spell].Level < 20 then
          healingbonus_penalty=(1-((20-HealBot_Spells[spell].Level)*0.0375));
        end
        local temp_Spell_cast=3.5;
        if HealBot_Spells[spell].Cast == 0 then
          temp_Spell_cast=3.5;
        end
        if HealBot_Spells[spell].Cast >= 1.5 and HealBot_Spells[spell].Cast < 3.5 then
          temp_Spell_cast=HealBot_Spells[spell].Cast;
        end
        RealHealing = ((HealBot_GetBonus() * healingbonus_penalty) * (temp_Spell_cast/3.5));
        local playerClass, englishClass = UnitClass("player");
        local SpiBonus = 0;
        if (englishClass=="PRIEST") then
          SpiBonus = ((HealBot_SpiBonus(spell) * healingbonus_penalty) * (temp_Spell_cast/3.5))
          RealHealing = RealHealing + SpiBonus;
        end
        RealHealing = floor(RealHealing);
        HealBot_Spells[spell].RealHealing = RealHealing;
      end
      HealBot_AddHeal(spell);
    end
  end);

  local items = {};
  for bag=0,NUM_BAG_FRAMES do
    for slot=1,GetContainerNumSlots(bag) do
      local item = HealBot_GetItemName(bag,slot);
      if HealBot_Spells[item] and not items[item] then
        HealBot_SetItemDefaults(item);
        HealBot_AddHeal(item);
        items[item] = 1;
      end
    end
  end
  table.foreach(HealBot_Heals, function (key,val)
    if (table.getn(val)==0) then
      HealBot_Heals[key] = nil;
    end
  end);
  HealBot_Heals.target = HealBot_Heals.party;
  for i=1,4 do
    HealBot_Heals["party"..i] = HealBot_Heals.party;
    HealBot_Heals["partypet"..i] = HealBot_Heals.party;
  end
  for i=1,40 do
    HealBot_Heals["raid"..i] = HealBot_Heals.party;
    HealBot_Heals["raidpet"..i] = HealBot_Heals.party;
  end

  table.foreach(HealBot_Heals, function (key,val)
    table.foreachi(val, function (i,val)
    end);
  end);
  if CalcEquipBonus then
    HealBot_AddChat("...Done Equip Bonus:"..RealHealing);
  end
  CalcEquipBonus=false;
end


function HealBot_GetShapeshiftForm()
  local forms = GetNumShapeshiftForms();
  if forms then
    local i;
    for i=1,forms do
      local icon,name,active = GetShapeshiftFormInfo(i);
      if active and not string.find(icon,"HumanoidForm") then return i; end
    end
  end
  return nil;
end

function HealBot_CanCastSpell(spell,unit)
  local this = HealBot_Spells[spell];
  if not this then return false end;
  if this.Mana>UnitMana("player") then return false end;
  if this.Level>UnitLevel(unit)+10 then return false end;
  if UnitOnTaxi("player") then return false end;
  if HealBot_UnitClass("player")=="DRUID" and HealBot_GetShapeshiftForm() then return false end;
  if HealBot_UnitAffected(unit,this.Buff) then return false end;
  if HealBot_UnitAffected(unit,this.Debuff) then return false end;
  if HealBot_ProtectPvP==1 and UnitIsPVP(unit) and not UnitIsPVP("player") then return false end
  if this.BagSlot then
    local bag,slot = HealBot_UnpackBagSlot(this.BagSlot);
    local start, duration, enable = GetContainerItemCooldown(bag,slot);
    if (start > 0 and duration > 0 and enable > 0) then
      return false;
    end
  end
  if HealBot_Range_Check(unit, this.range)==0 then return false end;
  return true;
end


function HealBot_GetHealSpell(unit,pattern)
  if (not UnitName(unit)) then return nil end;
  if pattern then
    return HealBot_GetSpellName(HealBot_GetSpellId(pattern))
  end
  if (not HealBot_Heals[unit]) then return nil end
  local Current,Desired = HealBot_UnitHealth(unit)
  if pattern then Current = 0 end
  local Needed = Desired-Current;
  if (Needed<=0 and not HealBot_AlwaysHeal()) then return nil end
  local PlayerHealth,PlayerMana = UnitHealth('player'),UnitMana('player');
  local UnitDPS,UnitTTL = HealBot_UnitDPS(unit),HealBot_UnitTTL(unit);
  local PlayerDPS,PlayerTTH = HealBot_UnitDPS('player'),HealBot_UnitTTH('player');
  local BestSpell = nil;
  local BestScore;
  local HealModifier;
  local HealUsage = floor(HealBot_HealingUsage) * 75;
  if HealUsage == 0 then 
     HealUsage = 1;
  end
  table.foreachi(HealBot_Heals[unit], function (i,spell)
    if pattern and not string.find(spell,pattern) then return end;
    local this = HealBot_Spells[spell];
    if (HealBot_CanCastSpell(spell,unit)) then
      local Health = Current+this.HealsDur-UnitDPS*this.Duration;
      if Health>Desired then
        Health = Desired;
      end
      if this.Shield then
        Health = Current;
        this.Duration = this.Shield;
        if UnitDPS*this.Duration>this.HealsDur then
          this.Duration = this.HealsDur/UnitDPS;
        end
      end
      local Score, n = 0, 0.1;
      local n = HealBot_ManaReserve;

      Score = Score + math.pow(Desired,n)/(1-n)*(math.pow(Health,(1-n))-math.pow(Current,(1-n)));
      Score = Score - math.pow(UnitManaMax('player'),n)/(1-n)*(math.pow(PlayerMana,(1-n))-math.pow((PlayerMana-this.Mana+1),(1-n)));

      Score = Score - (0+PlayerDPS)*this.Channel;
      Score = Score + UnitDPS*this.Duration;
      Score = Score - this.Price;
      if this.Mana>PlayerMana*(1-HealBot_ManaReserve) then
        Score = Score - 15000;
      end
      if this.BagSlot and this.Price==0 then
        Score = Score - this.HealsDur/100;
      end
      if this.BagSlot and UnitPowerType('player')==0 then
        Score = Score * (1-(PlayerMana/UnitManaMax('player')-0.4)*0.5);
      end
      if this.BagSlot and not HealBot_IsFighting then
          Score = Score - 5000;
      end
      local tmpHU=HealUsage;
      if tmpHU<0 then
         tmpHU=0-tmpHU;
      end
      if tmpHU<100 then
         tmpHU=100
      end
      local tmpTTL = UnitTTL;
      if tmpTTL > 50 then
         tmpTTL = 50;
      end
      if this.Cast then
         Score = Score - ((this.Cast*3) * (HealUsage / (tmpTTL/3)));
      end
      if tmpTTL<20 then
         if this.HealsMin == 0 then
            Score = Score - (tmpHU * (25-tmpTTL));
         end
      elseif this.BagSlot then
         if HealBot_HealingUsage > 0 then
            Score = Score - (tmpHU * (tmpTTL*9));
         else
            Score = Score - tmpHU; 
         end
      end
      if not BestSpell or Score>BestScore then
        BestSpell = spell;
        BestScore = Score;
      end
    end
  end);
  return BestSpell;
end

function HealBot_InitStatus()
  return { Health = 0, Damage = {}, DPS = 0, TTH = 10 };
end

HealBot_UnitStatus = {
  player = HealBot_InitStatus(),
  pet    = HealBot_InitStatus(),
  target = HealBot_InitStatus(),
};

for i=1,4 do
  HealBot_UnitStatus["party"..i] = HealBot_InitStatus();
  HealBot_UnitStatus["partypet"..i] = HealBot_InitStatus();
end
for i=1,40 do
  HealBot_UnitStatus["raid"..i] = HealBot_InitStatus();
  HealBot_UnitStatus["raidpet"..i] = HealBot_InitStatus();
end

function HealBot_UpdateDPS(unit)
  local now = GetTime();
  while HealBot_UnitStatus[unit].Damage[1] and HealBot_UnitStatus[unit].Damage[1].Time<now-60.0  do
    table.remove(HealBot_UnitStatus[unit].Damage,1);
  end
  local DPS,TTH = 0,0;
  table.foreachi(HealBot_UnitStatus[unit].Damage, function (i,val) 
    DPS = DPS + val.Amount*math.exp((val.Time-now)/HEALBOT_DPS_HORIZON);
    TTH = TTH + 1*math.exp((val.Time-now)/HEALBOT_DPS_HORIZON);
  end);
  DPS = DPS/HEALBOT_DPS_HORIZON;
  TTH = TTH/HEALBOT_DPS_HORIZON;
  if TTH<0.05 then TTH = 0.05 end
  HealBot_UnitStatus[unit].DPS = math.floor(10*DPS+0.5)/10;
  HealBot_UnitStatus[unit].TTH = math.floor(10*0.5/TTH+0.5)/10;
end

function HealBot_UpdateHealth(unit)
  if (UnitHealth(unit)<HealBot_UnitStatus[unit].Health) then
    table.insert(HealBot_UnitStatus[unit].Damage,
      {Time = GetTime(), Amount = HealBot_UnitStatus[unit].Health-UnitHealth(unit)});
  end
  HealBot_UnitStatus[unit].Health = UnitHealth(unit);
  HealBot_UpdateDPS(unit);
end

function HealBot_UpdateReset(unit)
  HealBot_UnitStatus[unit] = HealBot_InitStatus();
  HealBot_UpdateHealth(unit);
end

HEALBOT_DPS_HORIZON = 5.0;

function HealBot_UnitDPS(unit)
  if unit=='target' then return 0; end
  if HealBot_UnitAffected(unit,HEALBOT_EFFECT_POWER_WORD_SHIELD) then return 0; end
  return HealBot_UnitStatus[unit].DPS;
end

function HealBot_UnitTTL(unit)
  local dps = HealBot_UnitDPS(unit);
  local health = UnitHealth(unit);
  if (dps>0 and health>1) then
    return math.floor(health/dps+0.5);
  else
    return 18000;  -- you probably die within an 5 hours anyway :-)
  end
end

function HealBot_UnitTTH(unit)
  if unit=='target' then return 10; end
  if HealBot_UnitAffected(unit,HEALBOT_EFFECT_POWER_WORD_SHIELD) then return 10; end
  return HealBot_UnitStatus[unit].TTH;
end

function HealBot_HealUnit(unit,pattern)
  HealBot_CastSpellOnFriend(HealBot_GetHealSpell(unit,pattern),unit);
end

function HealBot_RecalcHeals(unit)
  HealBot_Action_Refresh(unit);
end

function HealBot_RecalcParty()
  HealBot_Action_PartyChanged();
  HealBot_Action_RefreshButtons();
end

function HealBot_RecalcSpells()
  HealBot_FindHealSpells();
  HealBot_RecalcParty();
end

--------------------------------------------------------------------------------------------------
-- OnFoo functions
--------------------------------------------------------------------------------------------------

function HealBot_OnLoad(this)
  this:RegisterEvent("VARIABLES_LOADED");
--  this:RegisterEvent("PLAYER_ENTERING_WORLD");
  
  SLASH_HEALBOT1 = "/healbot";
  SLASH_HEALBOT2 = "/hb";
  SlashCmdList["HEALBOT"] = function(msg)
    HealBot_SlashCmd(msg);
  end
  HealBot_AddError(HEALBOT_ADDON .. HEALBOT_LOADED);
end

function HealBot_RegisterThis(this)

end 



local HealBot_Timer1,HealsIn_Timer = 0,0;
function HealBot_OnUpdate(this,arg1)
  HealBot_Timer1 = HealBot_Timer1+arg1;
  if HealBot_Timer1>=1.0 then
    if not HealBot_IsFighting then
      HealsIn_Timer=HealsIn_Timer+1;
	  if HealsIn_Timer>=4000 then
        HealBot_HealsIn={};
	    HealsIn_Timer=0;
	  end
	else
	  HealsIn_Timer=0;
	end
    table.foreach(HealBot_UnitStatus, function (unit,status) 
      HealBot_UpdateDPS(unit);
    end)
    HealBot_Timer1 = 0;
    HealBot_Action_RefreshTooltip();
	if FlagEquipUpdate1>0 and FlagEquipUpdate2>0 then
	  FlagEquipUpdate1=0;
	  FlagEquipUpdate2=0;
	  NeedEquipUpdate=1;
	elseif FlagEquipUpdate1>0 then
	  FlagEquipUpdate1=FlagEquipUpdate1+1;
	  if FlagEquipUpdate1>3 then
	    FlagEquipUpdate1=0;
	  end
	elseif FlagEquipUpdate2>0 then
	  FlagEquipUpdate2=FlagEquipUpdate2+1;
	  if FlagEquipUpdate2>3 then
	    FlagEquipUpdate2=0;
	  end
	end
    if NeedEquipUpdate>0 then
      NeedEquipUpdate=NeedEquipUpdate+1;
      if NeedEquipUpdate>7 then
        HealBot_BonusScanner:ScanEquipment()
        CalcEquipBonus=true;
        HealBot_RecalcSpells();  
        NeedEquipUpdate=0; 
		end
    end
  end
end

function HealBot_OnEvent(this, event, arg1,arg2,arg3,arg4)
  if (event=="CHAT_MSG_ADDON") then
    HealBot_OnEvent_AddonMsg(this,arg1,arg2,arg3,arg4);
  elseif (event=="UNIT_HEALTH") then
    HealBot_OnEvent_UnitHealth(this,arg1);
  elseif (event=="UNIT_MANA") then
    HealBot_OnEvent_UnitMana(this,arg1);
--  elseif (event=="UNIT_AURA") then
--    HealBot_OnEvent_UnitAura(this,arg1);
  elseif (event=="SPELLCAST_START") then
    HealBot_OnEvent_SpellcastStart(this);
  elseif (event=="SPELLCAST_STOP") then
    HealBot_OnEvent_SpellcastStop(this);
  elseif (event=="SPELLCAST_INTERRUPTED") then
    HealBot_OnEvent_SpellcastStop(this);
  elseif (event=="SPELLCAST_FAILED") then
    HealBot_OnEvent_SpellcastStop(this);
  elseif (event=="PLAYER_REGEN_DISABLED") then
    HealBot_OnEvent_PlayerRegenDisabled(this);
  elseif (event=="PLAYER_REGEN_ENABLED") then
    HealBot_OnEvent_PlayerRegenEnabled(this);
  elseif (event=="BAG_UPDATE_COOLDOWN") then
    HealBot_OnEvent_BagUpdateCooldown(this,arg1);
  elseif (event=="BAG_UPDATE") then
    HealBot_OnEvent_BagUpdate(this,arg1);
  elseif (event=="PARTY_MEMBER_DISABLE") then
    HealBot_OnEvent_PartyMemberDisable(this,arg1);
--  elseif (event=="PARTY_MEMBER_ENABLE") then
--    HealBot_OnEvent_PartyMemberEnable(this,arg1);
  elseif (event=="PARTY_MEMBERS_CHANGED") then
    HealBot_OnEvent_PartyMembersChanged(this);
  elseif (event=="PLAYER_TARGET_CHANGED") then
    HealBot_OnEvent_PlayerTargetChanged(this);
  elseif (event=="ZONE_CHANGED_NEW_AREA") then
    HealBot_OnEvent_ZoneChanged(this);
  elseif (event=="UPDATE_INVENTORY_ALERTS") then
    HealBot_OnEvent_PlayerEquipmentChanged(this);
  elseif (event=="UNIT_INVENTORY_CHANGED") then
    HealBot_OnEvent_PlayerEquipmentChanged2(this,arg1);
  elseif (event=="PET_BAR_SHOWGRID") then
    HealBot_OnEvent_PartyMembersChanged(this);
  elseif (event=="PET_BAR_HIDEGRID") then
    HealBot_OnEvent_PartyMembersChanged(this);
--  elseif (event=="SPELLS_CHANGED") then
--    HealBot_OnEvent_SpellsChanged(this);
--  elseif (event=="PLAYER_ENTERING_WORLD") then
--    HealBot_OnEvent_PlayerEnteringWorld(this);
  elseif (event=="CHARACTER_POINTS_CHANGED") then
    HealBot_OnEvent_TalentsChanged(this, arg1);
  elseif (event=="VARIABLES_LOADED") then
    HealBot_OnEvent_VariablesLoaded(this);
  else
    HealBot_AddDebug("OnEvent (" .. event .. ")");
  end
end

function HealBot_OnEvent_VariablesLoaded(this)

  table.foreach(HealBot_ConfigDefaults, function (key,val)
    if not HealBot_Config[key] then
      HealBot_Config[key] = val;
    end
  end);
  
  HealBot_InitData();

  HealBot_BonusScanner:ScanEquipment();

  if HealBot_ActionVisible==1 then HealBot_Action:Show() end

  this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
  this:RegisterEvent("PLAYER_REGEN_DISABLED");
  this:RegisterEvent("PLAYER_REGEN_ENABLED");
  this:RegisterEvent("PLAYER_TARGET_CHANGED");
  this:RegisterEvent("PARTY_MEMBERS_CHANGED");
  this:RegisterEvent("PARTY_MEMBER_DISABLE");
--  this:RegisterEvent("PARTY_MEMBER_ENABLE");
  this:RegisterEvent("PET_BAR_SHOWGRID");
  this:RegisterEvent("PET_BAR_HIDEGRID");
  this:RegisterEvent("UNIT_HEALTH");
  this:RegisterEvent("UNIT_MANA");
--  this:RegisterEvent("SPELLS_CHANGED");
  this:RegisterEvent("SPELLCAST_START");
  this:RegisterEvent("SPELLCAST_STOP");
  this:RegisterEvent("SPELLCAST_INTERRUPTED");
  this:RegisterEvent("SPELLCAST_FAILED");
  this:RegisterEvent("BAG_UPDATE");
  this:RegisterEvent("BAG_UPDATE_COOLDOWN");
--  this:RegisterEvent("UNIT_AURA");
  this:RegisterEvent("CHARACTER_POINTS_CHANGED");
  this:RegisterEvent("UPDATE_INVENTORY_ALERTS");
  this:RegisterEvent("UNIT_INVENTORY_CHANGED");
  this:RegisterEvent("CHAT_MSG_ADDON");

end

function HealBot_OnEvent_AddonMsg(this,addon_id,inc_msg,dist_target,sender_id)
  if addon_id==HEALBOT_ADDON_ID then
    local tmpTest, unitname, heal_val
	tmpTest,tmpTest,unitname,heal_val = string.find(inc_msg, ">> (%a+) <<=>> (.%d+) <<" );
	if not heal_val then
      HealBot_AddChat("Addon Msg In (heals_val=nil): msg="..inc_msg.." Sender="..sender_id);
	else
--      HealBot_AddChat("Addon Msg In: Healing "..unitname.." for "..heal_val.." Sender="..sender_id);
      if not HealBot_HealsIn[unitname] then
		HealBot_HealsIn[unitname]=0;
      end
      HealBot_Healers[sender_id] = ">> "..unitname.." <<=>> "..heal_val.." <<";
      HealBot_HealsIn[unitname] = HealBot_HealsIn[unitname] + tonumber(heal_val);
      if HealBot_HealsIn[unitname] > 0 then
        HealBot_RecalcHeals(HealBot_FindUnitID(unitname))
	  elseif HealBot_HealsIn[unitname] < 0 then
	    HealBot_HealsIn[unitname]=0;
      end
    end
  elseif addon_id=="HealBot" then
    if inc_msg=="RequestVersion" then
      	SendAddonMessage( "HealBot", ">> SendVersion <<=>> Version="..HEALBOT_VERSION, "RAID" );
    else
      local tmpTest, datatype, datamsg
      tmpTest,tmpTest,datatype,datamsg = string.find(inc_msg, ">> (%a+) <<=>> (.+)" );
      if datatype=="SendVersion" then
        HealBot_AddChat(sender_id..":  "..datamsg);
      end
    end
  end
end
	
  
function HealBot_OnEvent_UnitHealth(this,unit)
    if (not HealBot_Heals[unit]) then return end
    HealBot_CheckCasting(unit);
    HealBot_RecalcHeals(unit);
    HealBot_UpdateHealth(unit);
end

function HealBot_OnEvent_UnitMana(this,unit)
  if (unit~="player") then return end
  HealBot_CheckCasting();
  HealBot_RecalcHeals();
end

function HealBot_OnEvent_ZoneChanged(this)
--   HealBot_AddChat("HB: ZoneChange");
   _scale = 0;
end

--function HealBot_OnEvent_UnitAura(this,unit)
--  if (not HealBot_Heals[unit]) then
--    return;
--  end
--  HealBot_RecalcHeals(unit);
--end

function HealBot_OnEvent_PlayerRegenDisabled(this)
  HealBot_IsFighting = true;
--  HealBot_RecalcHeals();
end

function HealBot_OnEvent_PlayerRegenEnabled(this)
  HealBot_IsFighting = false;
--  HealBot_RecalcHeals();
end

function HealBot_OnEvent_PlayerTargetChanged(this)
  HealBot_UpdateReset("target");
  HealBot_RecalcParty();
end

function HealBot_OnEvent_PartyMembersChanged(this)
  HealBot_RecalcParty();
end

function HealBot_OnEvent_PartyMemberDisable(this,unit)
  HealBot_AddChat(unit.." Disabled ");
  if string.len(unit) > 4 then
    local text = UnitName(unit);
    HealBot_AddChat(text.." Disabled ");
    if HealBot_Healers[text] then
      local tmpTest, unitname, heal_val, heal_valn
      tmpTest,tmpTest,unitname,heal_val = string.find(HealBot_Healers[text], ">> (%a+) <<=>> (.%d+) <<" );
      heal_valn=tonumber(heal_val)
      HealBot_AddChat("Healer "..text.." Disabled - Last known activity was heal "..unitname.." for "..heal_val);
      if heal_valn>0 and HealBot_HealsIn[unitname] then
        HealBot_HealsIn[unitname] = HealBot_HealsIn[unitname] - heal_valn;
        if HealBot_HealsIn[unitname] < 0 then
	      HealBot_HealsIn[unitname]=0;
        end
      end
    end
  end
--  HealBot_RecalcParty();    -- commented above as well for event
end

function HealBot_OnEvent_PartyMemberEnable(this,unit)
  HealBot_AddChat(unit.." Enabled ");
  local text = UnitName(unit);
  HealBot_AddChat(text.." Enabled ");
--  HealBot_RecalcParty();    -- commented above as well for event
end

function HealBot_OnEvent_PlayerEquipmentChanged(this)
  FlagEquipUpdate1=1;
end

function HealBot_OnEvent_PlayerEquipmentChanged2(this,unit)
  if unit=="player" then
    FlagEquipUpdate2=1;
  end
end

function HealBot_OnEvent_TalentsChanged(this, arg1)
  HealBot_AddChat("HB: TalentsChanged");
  initSpells=true;
end

function HealBot_OnEvent_SpellsChanged(this)
--  initSpells=true;    -- commented above as well for event
--  HealBot_RecalcSpells();
end

function HealBot_OnEvent_BagUpdate(this,bag)
  if FlagEquipUpdate1==0 and FlagEquipUpdate2==0 then
    HealBot_RecalcSpells();
  end
end

function HealBot_OnEvent_BagUpdateCooldown(this,bag)
  if not bag then 
    bag = "undef"
  elseif FlagEquipUpdate1==0 and FlagEquipUpdate2==0 then
    HealBot_RecalcSpells();
  end
end

function HealBot_OnEvent_PlayerEnteringWorld(this)
--   initSpells=true;    -- commented above as well for event
--   HealBot_RecalcSpells();
end

function HealBot_OnEvent_SpellcastStart(this)
  HealBot_IsCasting = true;
--  HealBot_RecalcHeals();
end

function HealBot_OnEvent_SpellcastStop(this)
  HealBot_IsCasting = false;
  HealBot_StopCasting();
  if HealBot_AbortButton==0 then
    HealBot_AbortButton=1;
    HealBot_Action_PartyChanged();
  end
--  HealBot_RecalcHeals();
end

function HealBot_SpiBonus(spell)
	local heals_modifer = 0;
    local base, stat, posBuff, negBuff = UnitStat("player",5);
	nameTalent, icon, tier, column, currRank, maxRank = GetTalentInfo(2,14); -- Spiritual guidence
	spiGuideBonus = stat * 0.05;
	heals_modifer = heals_modifer + (currRank * spiGuideBonus);
  	return heals_modifer;
end

function HealBot_GetBonus()
  local HealBonus=HealBot_BonusScanner:GetBonus();
  return HealBonus;
end

function HealBot_FindUnitID(unitname)
  local text;
  for _,unit in ipairs(HealBot_Action_HealGroup) do
    text = UnitName(unit);
	if text then
	  if text==unitname then
	    return unit;
	  end
	end
  end
  for i=1,40 do
    local unit = "raid"..i;
	text = UnitName(unit);
	if text then
      if text==unitname then
        return unit;
      end
	end
  end
  return nil;
end

function HealBot_InitSpells()
  local id = 1
  local cnt = 0;
  HealBot_CurrentSpells = {};
  while true do
    local spell = HealBot_GetSpellName(id);
    if not spell then
      do break end
    end
	if HealBot_IsCasting then
	  cnt=0;
	  do break end
	end
    if (HealBot_Spells[spell]) then
      HealBot_Spells[spell].id = id;
      HealBot_InitGetSpellData(spell, id, HealBot_UnitClass("player"));
      table.insert(HealBot_CurrentSpells,spell);
      cnt = cnt + 1;
    end
    id = id + 1;
  end
  HealBot_AddChat("Initiated HealBot_CurrentSpells with ".. cnt .." Spells");
  return cnt;
end

function HealBot_InitData()
  HealBot_Options_HealBot_Profile_Change(HealBot_Config.HealBot_Profile);
end

function HealBot_InitGetSpellData(spell, id, class)

  local i, _mana, _cast, _HealsMin, _HealsMax, _HealsExt, _duration, _range, _shield, _channel;
  local tooltip = getglobal( "HealBot_ScanTooltip" );
  local tmpText, line, tmpTest

  if ( not spell ) then
    return;
  end

  HealBot_ScanTooltip:SetSpell( id, BOOKTYPE_SPELL );
  tmpText = getglobal("HealBot_ScanTooltipTextLeft2");
  if (tmpText:GetText()) then
    line = tmpText:GetText();
    tmpTest,tmpTest,_mana = string.find(line, HB_TOOLTIP_MANA ); 
  end

  tmpText = getglobal("HealBot_ScanTooltipTextRight2");
  if (tmpText:GetText()) then
    line = tmpText:GetText();
    tmpTest,tmpTest,_range = string.find(line, HB_TOOLTIP_RANGE ); 
  end  

  tmpText = getglobal("HealBot_ScanTooltipTextLeft3");
  castTime = nil;
  if (tmpText:GetText()) then
    line = tmpText:GetText();
    if ( line == HB_TOOLTIP_INSTANT_CAST ) then
      _cast = 0;
    elseif line == HB_TOOLTIP_CHANNELED then
	  _cast = 0;
	elseif ( tmpText ) then
      tmpTest,tmpTest,_cast = string.find(line, HB_TOOLTIP_CAST_TIME ); 
    end
  end

  tmpText = getglobal("HealBot_ScanTooltipTextLeft4");
  tmpTest = nil;
  if (tmpText:GetText()) then
    line = tmpText:GetText();
	if class == "PRIEST" then
      if strsub(spell, 0, 14) == strsub(HEALBOT_POWER_WORD_SHIELD, 0, 14) then
        tmpTest,tmpTest,_HealsMin,_shield = string.find(line, HB_SPELL_PATTERN_SHIELD );    
        _HealsExt=0;
	    _HealsMax=_HealsMin;
        if ( _shield == nil ) then
          HealBot_AddChat( '_Shield is NIL' );
          HealBot_AddChat( 'Spell: '..spell );
          HealBot_AddChat(line);
          HealBot_AddChat(  HB_SPELL_PATTERN_SHIELD );
        end 
      elseif strsub(spell, 0, 4) == strsub(HEALBOT_RENEW, 0, 4) then
        tmpTest,tmpTest,_HealsExt,tmpTest,_duration = string.find(line, HB_SPELL_PATTERN_RENEW );  
        _HealsMin=0;
        _HealsMax=0;
        if ( _HealsExt == nil ) then
          tmpTest,tmpTest,_HealsExt,_duration = string.find(line, HB_SPELL_PATTERN_RENEW1 );
        end
      elseif strsub(spell, 0, 9) == strsub(HEALBOT_LESSER_HEAL, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line,HB_SPELL_PATTERN_LESSER_HEAL); 
      elseif strsub(spell, 0, 9) == strsub(HEALBOT_GREATER_HEAL, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_GREATER_HEAL ); 
      elseif strsub(spell, 0, 9) == strsub(HEALBOT_FLASH_HEAL, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_FLASH_HEAL ); 
      elseif strsub(spell, 0, 4) == strsub(HEALBOT_HEAL, 0, 4) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_HEAL ); 
      end
    elseif class=="DRUID" then
      if strsub(spell, 0, 6) == strsub(HEALBOT_REGROWTH, 0, 6) then
        tmpTest,tmpTest,_HealsMin,_HealsMax,_HealsExt = string.find(line, HB_SPELL_PATTERN_REGROWTH );
        if ( tmpTest == nil ) then
          tmpTest,tmpTest,_HealsMin,_HealsMax,tmpTest,_HealsExt = string.find(line, HB_SPELL_PATTERN_REGROWTH1 );
        end
      elseif strsub(spell, 0, 9) == strsub(HEALBOT_REJUVENATION, 0, 9) then
        tmpTest,tmpTest,_HealsExt,_duration = string.find(line, HB_SPELL_PATTERN_REJUVENATION );  
        _HealsMin=0;
        _HealsMax=0;
        if ( _HealsExt == nil ) then
          tmpTest,tmpTest,_HealsExt,tmpTest,_duration = string.find(line, HB_SPELL_PATTERN_REJUVENATION1 );
        end
      elseif strsub(spell, 0, 7) == strsub(HEALBOT_HEALING_TOUCH, 0, 7) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_HEALING_TOUCH ); 
      end
	elseif class=="PALADIN" then
      if strsub(spell, 0, 9) == strsub(HEALBOT_HOLY_LIGHT, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_HOLY_LIGHT ); 
      elseif strsub(spell, 0, 9) == strsub(HEALBOT_FLASH_OF_LIGHT, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_FLASH_OF_LIGHT ); 
      end
	elseif class=="SHAMAN" then
      if strsub(spell, 0, 9) == strsub(HEALBOT_HEALING_WAVE, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_HEALING_WAVE ); 
      elseif strsub(spell, 0, 9) == strsub(HEALBOT_LESSER_HEALING_WAVE, 0, 9) then
        tmpTest,_HealsMin,_HealsMax = HealBot_Generic_Patten(line, HB_SPELL_PATTERN_LESSER_HEALING_WAVE ); 
      end
	elseif class=="HUNTER" then
      if strsub(spell, 0, 6) == strsub(HEALBOT_MEND_PET, 0, 6) then
        tmpTest,tmpTest,_HealsExt,_channel = string.find(line, HB_SPELL_PATTERN_MEND_PET );
        _HealsMin=0;
        _HealsMax=0;
		_HealsExt=_HealsExt*_channel;
      end
	end
  end

  if ( _mana == nil ) then
    HealBot_AddChat( "================================" );
    HealBot_AddChat( "ERROR: _mana is NIL" );
    HealBot_AddChat( "ERROR: Spell: "..spell );
	if HealBot_ScanTooltipTextLeft2:GetText() then
      HealBot_AddChat( "ERROR: Tooltip = >> "..HealBot_ScanTooltipTextLeft2:GetText().." <<" );
    end
    HealBot_AddChat( "ERROR: Patter = >> "..HB_TOOLTIP_MANA.." <<" );
    HealBot_AddChat( "================================" );
  end
  if ( _cast == nil ) then
    HealBot_AddChat( "================================" );
    HealBot_AddChat( "ERROR: _cast is NIL" );
    HealBot_AddChat( "ERROR: Spell: "..spell );
	if HealBot_ScanTooltipTextLeft3:GetText() then
      HealBot_AddChat( "ERROR: Tooltip = >> "..HealBot_ScanTooltipTextLeft3:GetText().." <<" );
    end
    HealBot_AddChat( "ERROR: Patter = >> "..HB_TOOLTIP_CAST_TIME.." <<" );
    HealBot_AddChat( "================================" );
  end  
  if ( tmpTest == nil ) then
    HealBot_AddChat( "================================" );
    HealBot_AddChat( "ERROR: tmpTest == nil" );
    HealBot_AddChat( "ERROR: spell = "..spell );
    if line then
      HealBot_AddChat( "ERROR: Tooltip = >> "..line.." <<" );
    end
    HealBot_AddChat( "================================" );
  end  
  
  HealBot_Spells[spell].Cast=tonumber(_cast);
  HealBot_Spells[spell].Mana=tonumber(_mana);
  HealBot_Spells[spell].Range=tonumber(_range);
  HealBot_Spells[spell].HealsMin=tonumber(_HealsMin);
  HealBot_Spells[spell].HealsMax=tonumber(_HealsMax);
  if _HealsExt then
    HealBot_Spells[spell].HealsExt=tonumber(_HealsExt);
  end
  if _duration then
    HealBot_Spells[spell].Duration=tonumber(_duration);
  end
  if _shield then
    HealBot_Spells[spell].Shield=tonumber(_shield);
  end
  if _channel then
    HealBot_Spells[spell].Channel=tonumber(_channel);
  end
  
  if not HealBot_Spells[spell].Target then
    HealBot_Spells[spell].Target = {"player","party","pet"};
  end
  if not HealBot_Spells[spell].Price then
    HealBot_Spells[spell].Price = 0;
  end
  if not HealBot_Spells[spell].Channel then
    HealBot_Spells[spell].Channel = HealBot_Spells[spell].Cast;
  end
  if not HealBot_Spells[spell].Duration then
    HealBot_Spells[spell].Duration = HealBot_Spells[spell].Channel;
  end
  if not HealBot_Spells[spell].RealHealing then
    HealBot_Spells[spell].RealHealing=0;
  end
  HealBot_Spells[spell].HealsCast = (HealBot_Spells[spell].HealsMin+HealBot_Spells[spell].HealsMax)/2;
  if not HealBot_Spells[spell].HealsExt then
    HealBot_Spells[spell].HealsExt = 0;
  end

end

function HealBot_Generic_Patten(matchStr,matchPattern)
  local tmpTest,_HealsMin,_HealsMax,_HealsExt,_duration
    tmpTest,tmpTest,_HealsMin,_HealsMax = string.find(matchStr, matchPattern ); 
  if ( tmpTest == nil ) then
    HealBot_AddChat( "================================" );
    HealBot_AddChat( "ERROR: tmpTest == nil" );
    HealBot_AddChat( "ERROR: pattern = "..matchPattern );
    HealBot_AddChat( "ERROR: Tooltip = >> "..matchStr.." <<" );
    HealBot_AddChat( "================================" );
  end
  return tmpTest,_HealsMin,_HealsMax;
end

function HealBot_Get_DebugChan()
	local index = GetChannelName("HBmsg");
	if (index>0) then
		return index;
	else
		return nil;
	end
end

function HealBot_Range_Check(unit, range)

  local return_val = 0;
  if not range then 
    range=40;
  end
  range = range * 0.98;
  if ( unit=="player" ) then 
    return_val = 1;
  elseif ( UnitIsVisible(unit) == 1) then
    local tx, ty = GetPlayerMapPosition(unit)
    local dist
    if tx > 0 or ty > 0 then
      local px, py = GetPlayerMapPosition("player")
      dist = sqrt((px - tx)^2 + (py - ty)^2)
      if dist > _scale and (px > 0 or py > 0) then
        if (CheckInteractDistance(unit, 4)) then
          _scale = dist
        end
      end
      if dist <=(_scale*range/27) then
        return_val=1
      end
    else
      if (HealBot_QualityRange == 1) or range < 28 then
        if ( CheckInteractDistance(unit, 4) ) then
          return_val = 1;
        end                    
      else
        return_val = 1;
      end
    end
  end
  return return_val;
end